==> mpc_0.25.orig.tar.xz: mpc-0.25/src/list.h <==
/*
 * music player command (mpc)
 * Copyright (C) 2003-2013 The Music Player Daemon Project
 * http://www.musicpd.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef LIST_H
#define LIST_H

#include <stdlib.h>

/* used to make a list where free() will be used to free data in list */
#define DEFAULT_FREE_DATA_FUNC	free

/* typedef for function to free data stored in the list nodes */
typedef void ListFreeDataFunc(void *);

typedef struct _ListNode {
	/* used to identify node (ie. when using findInList) */
	char * key;
	/* data store in node */
	void * data;
	/* next node in list */
	struct _ListNode * nextNode;
	/* previous node in list */
	struct _ListNode * prevNode;
} ListNode;

typedef struct _List {
	/* first node in list */
	ListNode * firstNode;
	/* last node in list */
	ListNode * lastNode;
	/* function used to free data stored in nodes of the list */
	ListFreeDataFunc * freeDataFunc;
	/* number of nodes */
	long numberOfNodes;
	/* array for searching when list is sorted */
	ListNode ** nodesArray;
} List;

/* allocates memory for a new list and initializes it
 *  _freeDataFunc_ -> pointer to function used to free data, use 
 *                    DEFAULT_FREE_DATAFUNC to use free()
 * returns pointer to new list if successful, NULL otherwise
 */
List * makeList(ListFreeDataFunc * freeDataFunc);

/* inserts a node into _list_ with _key_ and _data_
 *  _list_ -> list the data will be inserted in
 *  _key_ -> identifier for node/data to be inserted into list
 *  _data_ -> data to be inserted in list
 * returns 1 if successful, 0 otherwise
 */ 
int insertInList(List * list,char * key,void * data);

int insertInListWithoutKey(List * list,void * data);

/* deletes the first node in the list with the key _key_
 *  _list_ -> list the node will be deleted from
 *  _key_ -> key used to identify node to delete
 *  returns 1 if node is found and deleted, 0 otherwise
 */
int deleteFromList(List * list,char * key);

void deleteNodeFromList(List * list,ListNode * node);

/* finds data in a list based on key
 *  _list_ -> list to search for _key_ in
 * _key_ -> which node is being searched for
 * _data_ -> a pointer to where data will be placed, 
 *	_data_ memory should not by allocated or freed
 * returns 1 if successful, 0 otherwise
 */
int findInList(List * list, char * key, void ** data);

/* frees memory malloc'd for list and its nodes
 *  _list_ -> List to be free'd
 */
void freeList(void * list);

void clearList(List * list);

#endif

==> mpc_0.28.orig.tar.xz: mpc-0.28/src/list.h <==
/*
 * music player command (mpc)
 * Copyright (C) 2003-2015 The Music Player Daemon Project
 * http://www.musicpd.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef LIST_H
#define LIST_H

typedef struct _ListNode {
	/* data store in node */
	void * data;
	/* next node in list */
	struct _ListNode * nextNode;
} ListNode;

typedef struct _List {
	/* first node in list */
	ListNode * firstNode;
	/* last node in list */
	ListNode * lastNode;
	/* number of nodes */
	long numberOfNodes;
} List;

/* allocates memory for a new list and initializes it
 *  _freeDataFunc_ -> pointer to function used to free data, use 
 *                    DEFAULT_FREE_DATAFUNC to use free()
 * returns pointer to new list if successful, NULL otherwise
 */
List *makeList(void);

void insertInListWithoutKey(List * list,void * data);

/* frees memory malloc'd for list and its nodes
 *  _list_ -> List to be free'd
 */
void freeList(void * list);

#endif
